# ShowDisks.py
""" A module for getting practice with classes and objects.
"""

from math import pi, sqrt
from simpleGraphicsE import *
from random import randint as randi
from random import uniform as randu


class Point:
    """
    Attributes:
        x: float, the x-coordinate of a point
        y: float, the y-coordinate of a point
    """
    def __init__(self,x,y):
        """ Creates a Point object
        PreC: x and y are floats
        """
        self.x = x
        self.y = y
    
class Disk:
    """
    Attributes:
        center: Point, the center of the disk
        radius: float, the radiusof the disk
        area: float, the area of the disk
        circ: float, the  circumference of the disk
        
    """
    def __init__(self,P,r):
        """ Creates a Disk object with center P and radius r
        PreC: P is a Point and r is a positive float
        """
        self.center = P
        self.radius = r
        self.area = pi*r**2
        self.circ = 2*pi*r

def theyTouch(D1,D2):
    """ Returns true if D1 and D2 intersect and false otherwise
    PreC: D1 and D2 are disks
    """
    # The center-to-center distance:
    dD1toD2 = Dist(D1.center,D2.center)
    # The sum of the two radii
    radiusSum = D1.radius + D2.radius
    return radiusSum>=dD1toD2

def outsideAll(D0,L):
    """ Returns True if D0 doesn't touch any of the disks in L
    
    PreC: D0 is a Disk and L is a list of Disks
    """
    for D in L:
        if theyTouch(D0,D):
            return False
    return True
    

def RandomDisk(n):
    """ Returns a random disk whose center is inside
    the 2n-by2n square centered at (0,0).
    
    Pre: n is apositive int
    """
    P = RandomPoint(n)
    r = randu(0.5,1.0)*n/10.
    return Disk(P,r)
    
def ShowDisk(D):
    """ Displays D with equal chance of being magenta or cyan
    
    PreC: D is a Disk
    """
    if randi(0,1) == 0:
        c = MAGENTA
    else:
        c = CYAN
    # Get the xy cooordinates of the center
    xc = D.center.x
    yc = D.center.y
    DrawDisk(xc,yc,D.radius,color=c)
    
def RandomPoint(n):
    """ Returns a Point that is inside the 2n-by-2n square centered
    at (0,0).
    
    PreC: n is a positive int.
    """
    x = randu(-n,n)
    y = randu(-n,n)
    P = Point(x,y)
    return P

def Dist(P1,P2):
    """ Returns a float that is the distance from P1 to P2.
    
    PreC: P1 and P2 are points
    """
    d = sqrt((P1.x-P2.x)**2+(P1.y-P2.y)**2)
    return d

    
        
if __name__ == '__main__':
    n = 10
    MakeWindow(n,bgcolor=BLACK)
    # Solutions go here...
    
    
    
    
    
    ShowWindow()
    


