# SonnetTools.py
""" Helper functions to be used in Assignment 6.
"""

def GetSonnets():
    """ Returns a list of strings L with the property that
    L[k] is the kth line in the file Sonnets.txt.
    """
    L = []
    with open('TheSonnets.txt',"r") as F:
        for s in F:
            # Add the next line in the file to L without the trailing "\n" (the
            # new line character), and without the trailing "\r" (the carriage
            # return character)
            L.append(s.rstrip("\n").rstrip("\r"))
    return L

def dePunc(s):
    """Returns a string that is the same as s except that all occurrences of the
    period, comma, semicolon, colon, question mark, and explanation point are
    replaced by the null string. Leading and trailing apostrophes are also removed.
     
    Example: dePunc('a.b,c;d:e?f!') returns 'abcdef'
    
    PreC: s is a string.
    """
    if s == '':
        return s
    # s is not the empty string, remove punctuation
    s = s.replace('.','')
    s = s.replace(',','')
    s = s.replace(';','')
    s = s.replace(':','')
    s = s.replace('?','')
    s = s.replace('!','')
    if s[0]=='\'':
        s = s[1:]
    if s[len(s)-1]=='\'':
        s = s[:len(s)-1]
    return s
    
def theLastWord(s):
    """ Returns the last word in s where the last
    word in the string s.
    
    If t is the longest substring of the form s[k:] that
    does not contain a blank, then the string returned is
    t with all periods, commas, semicolons, colons, question marks,
    exclamation points, parentheses, and hypends removed. Leading
    and trailing apostrophes are also removed.
    
    Example: theLastWord('ab cdef ghk!') returns 'ghk'
    
    PreC: s is a string with at least one blank.
    """
    k = s.rfind(' ')
    t = s[k+1:].lower()
    t = dePunc(t)
    t = t.replace('(','')
    t = t.replace(')','')
    t = t.replace('-','')
    if t[0]=='\'':
        t = t[1:]
    if t[-1]=='\'':
        t = t[:-1]
    return t

def printSonnet(S):
    """ Displays the attributes of a sonnet object.
    PreC: S is a sonnet object.
    """
    print '\nSonnet %s\n' % S.index
    for k in range(14):
        if k>11:
           print '   '+S.text[k]
        else:
           print S.text[k]
    print '\nNumber of words = %1d' % S.nWords
    print '\nFirst Line: %s' % S.firstLine
    LW = S.lastWords
    print '\nLast words: %10s  %10s  %10s  %10s ' % (LW[0],LW[1],LW[2],LW[3])
    print '            %10s  %10s  %10s  %10s ' % (LW[4],LW[5],LW[6],LW[7])
    print '            %10s  %10s  %10s  %10s ' % (LW[8],LW[9],LW[10],LW[11])
    print '            %10s  %10s         ' % (LW[12],LW[13])
    
    
def get_firstLine(s):
    """ Returns the first line of the Sonnet object s.

    Purpose is for usage with SortSonnetList as the function being
    mapped with python's sorted method.

    PreC: s is a Sonnet object
    """
    return s.firstLine

def get_nWords(s):
    """ Returns the number of words in the Sonnet object s.

    Purpose is for usage with SortSonnetList as the function being
    mapped with python's sorted method.

    PreC: s is a Sonnet object
    """
    return s.nWords

def SortSonnetList(ListOfSonnets,getterF):
    """ Returns a permuted copy of ListOfSonnets so that it
    is sorted based on the getter function getterF.
    
    The list ListOfSonnets is not modified.
    
    PreC: ListOfSonnets is a list of sonnet objects.
    getterF must reference either get_firstLine, get_nWords,
    or get_nClauses.
    """
    
    return sorted(ListOfSonnets,key=getterF)

def printRhymeDict(D):
    """ Displays the keys and values of D.
    PreC: D is a rhyming dictionary.
    """
    print '\n'
    for d in sorted(D):
        s =  '%15s : ' % d
        for w in sorted(D[d]):
           s = s + w + ', '
        print s[:-2]

def saveRhymeDictToFile(D):
    """
    Saves the Rhyming Dictionary D to a text file named DictionaryValues.txt.

    PreC: D is a valid Rhyming Dictionary.  If D is empty, nothing will happen.
    """
    # verify we are guaranteed working with a populated dictionary
    if D == None or D == {} or not isinstance(D, dict):
        raise TypeError("\nYou must supply a Dictionary that has:\n" +
                             "     - strings for keys, and\n"             +
                             "     - lists of strings for values.\n")

    # some type safety is needed here beyond what checking for dictionaries
    # with duck typing will do, we need keys as strings, and values as
    # lists of strings for this to write everything correctly
    for k in D:
        vals = D[k]
        if not (isinstance(k, basestring) and \
                isinstance(vals, list)    and \
                all(isinstance(item, basestring) for item in vals)):
            raise ValueError("\nYou must supply a Dictionary that has:\n" +
                             "     - strings for keys, and\n"             +
                             "     - lists of strings for values.\n")

    with open("DictionaryValues.txt", "w") as f:
        # determine longest key for pretty printing
        longest = 0
        for k in D:
            if len(k) > longest:
                longest = len(k)

        # extent is used to write new lines of values the same
        # key, since we are writing 4 values per line. +3 comes
        # from the key being written as "<key> : "
        #                                     ^^^
        #                                      3 
        extent = ''
        for space in xrange(longest + 3):
            extent += ' '

        # write the keys in alphabetical order
        for k in sorted(D):
            # prepend spaces to key string
            pre_space = longest - len(k)
            key_string = ''
            for space in xrange(pre_space):
                key_string += ' '
            key_string += k + " : "
            # sys.stdout.write(key_string)
            f.write(key_string)

            # write all values associated with key
            matches = D[k]
            next = ''
            num = len(matches)
            for i in xrange(num):
                next += matches[i]
                if i < num-1:
                    if i%3 == 0 and i > 0: # 4 values per line
                        next += ",\n" + extent
                    else:
                        next += ", "
            # sys.stdout.write(next + "\n")
            f.write(next + "\n")    
